@echo off
REM Copy saved 64bit libraries into place, if present
REM iphlpapi.lib libeay32.dll libeay32.lib ssleay32.dll ssleay32.lib
REM Modify SAVLIB64 to a directory where these libs are saved.
set SAVLIB64=lib64
IF NOT EXIST %SAVLIB64% GOTO NOSAVLIB
copy /Y %SAVLIB64%\*.lib lib
copy /Y %SAVLIB64%\*.dll util
REM (do not copy mak) copy /Y util\x64\ipmiutil.mak util
:NOSAVLIB
REM sample VCINSTALLDIR from VS 2008
set VCDIR=C:\Program Files (x86)\Microsoft Visual Studio 9.0\VC
IF DEFINED VCINSTALLDIR GOTO HAVEVC
IF DEFINED VSINSTALLDIR GOTO HAVEVS
GOTO HAVEDEF
:HAVEVS
set VCDIR=%VSINSTALLDIR%\VC
:HAVEVC
set VCDIR=%VCINSTALLDIR%
:HAVEDEF
REM Run MS 64bit vcvars
set VCBAT="%VCDIR%\bin\x86_amd64\vcvarsx86_amd64.bat"
set VCBATDIR="C:\Program Files (x86)\Microsoft Visual Studio\2019\Community\VC\Auxiliary\Build"
set VCBAT=%VCBATDIR%\vcvarsx86_amd64.bat
IF EXIST %VCBAT% GOTO RUNVC
echo "Cannot locate vcvars for amd64, please run it manually."
GOTO DONE
:RUNVC
%VCBAT%
:DONE
